function [eppm, time, trials] = eppmatrix (c, ch, k)

% @cortex\eppmatrix.m
%
% returns a matrix of EPP values from a @cortex object, given a @key
%
% Syntax 1: [EPPM,TIME] = EYEPOS(COBJ,CH,KEY);
%
% where
% COBJ:  the @cortex object
% CH:    the channel number (currently 1 or 2 only)
% KEY:   one key from a @key object
% EPPM:  matrix of EPP values (one row per trial, one column per sample;
%        values are time-locked on the cue(s) specified in the key
% TIME:  the corresponding time scale
%
% Last modified: 28 Oct 04

% Created on 23 June 04
% 28 Oct 04: cosmetics

if isempty (c.epp), error ('object does not contain EPP data'); end
errifnotsinglekey (k);
switch ch
   case 1
      epps = c.epp.ch1;
   case 2
      epps = c.epp.ch2;
   otherwise
      error ('invalid channel number');
end
rate = c.epp.rate;
epps_per_trial = diff (c.epp.ptr);
trials_with_epps = find (epps_per_trial);
yestrials_with_epps = intersect (trials_with_epps, yestrials);
if length (yestrials_with_epps) < length (yestrials)
   warning ('among the requested trials, the following do not have EPPs:');
   disp (setdiff(yestrials,trials_with_epps));
end
samples_per_row = fix ((k.finish-k.start)/rate);
eppm = zeros (length (yestrials_with_epps), samples_per_row);
instance = 0;
trials = [];
cueless = [];
winless = [];

for trial=yestrials_with_epps
   first = c.epp.ptr(trial);
   last  = c.epp.ptr(trial+1)-1;
   samples = epps_per_trial(trial);
   % of all requested cues, find those that are actually present
   % in the current trial. THIS MAY BE SLOW!
   cues_found = intersect (k.cues, c.codes (trial, :));
   % if no cue event is found add trial to cueless
   if isempty (cues_found), cueless = [cueless trial]; end
   % for each cue event found
   for j=cues_found
      % there should NOT be more than one instance of the same cue event!
      cue_index = find (c.codes (trial, :) == j);
      if length (cue_index)>1
         disp (['only the first instance of event ', num2str(j), ' will be considered']);
         cue_index = cue_index(1);
      end
      
      kt = c.times (trial, cue_index);
      start_time = kt + k.start;
      end_time   = kt + k.finish;
      start_from_first = start_time - c.epp.start(trial);
      if (start_from_first < 0 | (end_time > (start_time + samples * rate)))
         winless = [winless trial];
      else
         instance = instance + 1;
         trials = [trials; trial]; 
         findx = first + fix (start_from_first/rate);
         lindx = findx + samples_per_row-1;
         eppm (instance, :) = double(epps (findx:lindx))';
      end
   end
end
eppm = eppm (1:instance, :);
time = k.start:rate:k.finish-rate;

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
if ~isempty (winless)
   warning ('could not retrieve all requested trials;');
   disp ('time window exceeds boundaries of the following trial(s)');
   disp (winless);
end