function k = key (label, start, finish, cues, conditions, blocks, responses, trials, type, given, rept, rel) 

% @KEY/KEY.M   the @key class constructor
%
% @key objects are used to select subsets of trials and analysis time
% windows from a @cortex object. See online help for more information
% on @key object usage.
%
% KEY(LABEL,START,FINISH,CUES,CONDS,BLOCKS,RESPS,TRIALS,TYPE,GIVEN,REPT,REL)  
%    creates and returns a single-key object and assigns the values
%    passed as arguments to the following corresponding fields:
%    LABEL:  the name of the key;
%    START
%    FINISH: the offset in ms from the cue event;
%    CUES:   the cue events, an array of encodes to time-lock analysis on; 
%
%    The next 8 arguments are arrays of values that restrict analyses to
%    subsets of trials (see online help for more information)
%
%    CONDS:  condition numbers
%    BLOCKS: block numbers
%    RESPS:  response_error(s) (i.e., the trial outcome)
%    TRIALS: "absolute" trial numbers
%    TYPE:   trial types (i.e., the expected response)
%    GIVEN:  given responses (aka the "response")
%    REPT:   repetition numbers
%    REL:    "relative" trial numbers (trial # recorded in trial header)
%
%    note that passing -1 is equivalent to including all possible
%    field values (e.g., if k.blocks == -1 then all blocks are
%    included in the  analysis performed by SPIKEMATRIX).
%
% KEY without arguments returns a single-key object and sets its fields to:
%    LABEL='noname', START=0, FINISH=300, CUES=23, RESPONSES=0;
%    CONDITIONS = BLOCKS = TRIALS = TYPE = GIVEN = REPT = REL = -1.
%
% KEY(KEYFILE) returns a multiple-key object created on the basis of
%    the KEYFILE text file. See online help for more information on
%    keyfile syntax)
%
% Last modified: 20 Nov 2004

% 30 jan 99: last major changes
% 18 apr 01: added little check to avoid warnings: if ~isempty (lin), if lin == -1 ...
% 14 may 01: branch off to @key class
% 10 oct 02: help text fixed
%  1 Dec 02: help text fixed
% 20 Nov 04: rel_trial added; expected changed to given;
%            help text revised


switch nargin
   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   case 0  % default object
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      k.size      =   1;
      k.label{1}  = 'noname';
      k.st.start  =   0;
      k.st.finish = 300;
      k.st.cues   =  23;
      k.st.conds  =  -1;
      k.st.blocks =  -1;
      k.st.resps  =   0;
      k.st.trials =  -1;
      k.st.type   =  -1;
      k.st.given =   -1;
      k.st.rept   =  -1;
      k.st.rel    =  -1;
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   case 12  % then use them all
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      k.size = 1;
      if isempty (label)
         k.label{1} = 'noname';
      else
         k.label{1} = label;
      end
      if isempty (start)
         k.st.start = -50;
      else
         k.st.start = start;
      end
      if isempty (finish)
         k.st.finish = 250;
      else
         k.st.finish = finish;
      end
      if isempty (cues)
         k.st.cues = 23;
      else
         k.st.cues = cues;
      end
      if isempty (conditions)
         k.st.conds = -1;   
      else
         k.st.conds = conditions;
      end
      if isempty (blocks)
         k.st.blocks = -1;
      else
         k.st.blocks = blocks;
      end
      if isempty (responses)
         k.st.resps = -1;
      else
         k.st.resps = responses;
      end
      if isempty (trials)
         k.st.trials = -1;
      else
         k.st.trials = trials;
      end
      if isempty (type)
         k.st.type = -1;
      else
         k.st.type = type;
      end
      if isempty (given)
         k.st.given = -1;
      else
         k.st.given = given;
      end
      if isempty (rept)
         k.st.rept = -1;
      else
         k.st.rept = rept;
      end
      if isempty (rel)
         k.st.rel = -1;
      else
         k.st.rel = rel;
      end
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   case 1 % the argument is a file name
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      [list, rows, nfields, empty] = listfromfile (label);
      if empty, error ('all rows in key source file must contain the same number of fields'); end
      if ~strcmp (list {1,1}, 'label'), error ('first field in key file must be ''label'''); end
      if rows<2, error ('key source file must contain at least a header and one key'); end
      
      fields = list(1,:);
      keys   = list(2:end,:);
      keys_per_group = 0;
      group          = 0;
      old_label      ='';
      total_keys = rows - 1;
      
      for ck = 1:total_keys
         label = keys{ck,1};
         errifnotstring (label, 'LABEL');
         % if the current key label is different from the previous
         % then this is a new group (also applies to the very first key)
         if ~strcmp (label, old_label)
            group = group + 1;
            k.size (group) = 0;
            k.label {group} = label;
         end
         k.size (group) = k.size (group) + 1;
         old_label = label;
         
         % every field in the current key is at first set to a default value
         k.st(ck).start  =   0;
         k.st(ck).finish = 300;
         k.st(ck).cues   =  23;
         k.st(ck).conds  =  -1;
         k.st(ck).blocks =  -1;
         k.st(ck).resps  =   0;
         k.st(ck).trials =  -1;
         k.st(ck).type   =  -1;
         k.st(ck).given  =  -1;
         k.st(ck).rept   =  -1;
         k.st(ck).rel    =  -1;
         
         % loop through the fields actually present in the file
         % except for 'label', which is compulsory
         
         for fi=2:nfields
            v = keys{ck,fi};
            if ~isnumvector (v), error ('field values in keyfile must be a numeric vector'); end
            % try to assign the value to a valid field
            switch (minifield(fields{fi}))
               case 'tri'
                  k.st(ck).trials = v;
               case 'con'
                  k.st(ck).conds = v;
               case 'blo'
                  k.st(ck).blocks = v;
               case 'res'
                  k.st(ck).resps = v;
               case 'typ'
                  k.st(ck).type  = v;
               case 'exp'
                  k.st(ck).given = v;
               case 'giv'
                  k.st(ck).given = v;
               case 'rep'
                  k.st(ck).rept = v;
               case 'rel'
                  k.st(ck).rel = v;
               case 'cue'
                  k.st(ck).cues = v;
               case 'sta'
                  errifnotscalar (v, 'START');
                  k.st(ck).start = v;
               case 'fin'
                  errifnotscalar (v, 'FINISH');
                  k.st(ck).finish = v;
               otherwise
                  error ('bad field name in header');
            end
         end % resume here after having processed all fields in oneline
      end
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   otherwise
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      error ('wrong number of arguments');
   end
   
   %finish up
   k = class (k, 'key');
